package Source::Repository;

use strict;
use warnings;
use Carp;

use File::Path qw(make_path remove_tree);
use File::Basename;
use Source::Package 0.183;
use Source::Shared::Resource::Verbose;
use Source::Shared::Utils::GlobList qw/_line2array/;
use Source::Repository::Chain;

our $VERSION=0.410;
our $verbose=0;

# protected; should not be called outside the module
sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	#-cachedir=>'in',
	@_,
    };
    bless($self,$class);
    # TODO:
    $self->__load_repolist();
    $self->__load_namelist();

    if ($verbose>4) {
	Carp::cluck "DEBUG:Source::Repository:new: class=$class\n-- keys:".join(',',%$self)."\n";
	#use Data::Dumper;
	#print STDERR Dumper($self);
    }
    return $self;
}

sub create_repo_chain_from_cli {
    my $proto=shift;
    my $class=ref($proto) || $proto;

    my $opts = {
	-reponame => __PACKAGE__,
	@_,
    };
    #use Data::Dumper;
    #print STDERR Dumper($opts);
    confess "deprecated -pathlist" if $opts->{-pathlist};
    my @cli_init_keys = grep {$_ eq '-path' or $_ eq '-default_opt'} keys(%$opts);
    warn $opts->{-reponame},': multiple initialization: ', join (',',@cli_init_keys),"\n" if @cli_init_keys>1;
    if (@cli_init_keys==0) {
	die $opts->{-reponame},': not initialized. You have to provide repository location';
	return $class->new(%$opts);
    } else {
	my @components;
	my @elements;
	if ($opts->{-path}) {
	    push @elements, @{$opts->{-path}};
	}
	if ($opts->{-default_opt}) {
	    push @elements, &__split_path_like_uri_by_colon($opts->{-default_opt});
	    print STDERR "DEBUG:create_repo_chain_from_cli: elements=@elements\n" if $verbose>2;
	}
	delete $opts->{-path};
	delete $opts->{-default_opt};
    
	my $namelist=$opts->{-namelist};
	delete $opts->{-namelist};
	foreach my $pathelement (@elements) {
	    if (-d $pathelement) {
		print STDERR "DEBUG:create_repo_chain_from_cli: from dir=$pathelement\n" if $verbose>2;
		push @components, $class->create_repo_from_dir($opts, $pathelement);
	    } elsif (-f $pathelement) {
		print STDERR "DEBUG:create_repo_chain_from_cli: from file=$pathelement\n" if $verbose>2;
		push @components, $class->create_repo_from_file($opts, $pathelement);
	    } elsif ($pathelement=~m!^(https?|ftp|rsync)://!) {
		print STDERR "DEBUG:create_repo_chain_from_cli: from url=$pathelement\n" if $verbose>2;
		push @components, $class->create_repo_from_url($opts, $pathelement);
	    } else {
		warn $opts->{-reponame},": not found: $pathelement\n" if $verbose or $Source::Shared::verbose;
	    }
	}
	unless (@components) {
	    die $opts->{-reponame},': repository not found in ',join(':',@elements),"\n";
	}
	my @ARG=(%$opts, -components=>\@components);
	push @ARG, -namelist => $namelist if $namelist;
	return Source::Repository::Chain->new(@ARG);
    }
}

sub get_package_class {
    return 'Source::Package';
}

# protected; should not be called outside the module
sub create_repo_from_dir {
    my ($class, $opt, $uri)=@_;
    die "Oops! Source::Repository: create_repo_from_dir: not implemented";
    #return $class->new(%$opt, -path => [$uri]);
}

# protected; should not be called outside the module
sub create_repo_from_file {
    my ($class, $opt, $uri)=@_;
    # die "Oops! Source::Repository: create_repo_from_file: not implemented";
    return $class->new(%$opt, -repolist => $uri, -created_from => $uri );
}

# protected; should not be called outside the module
sub create_repo_from_url {
    my ($class, $opt, $uri)=@_;
    die "Oops! Source::Repository: create_repo_from_url: not implemented";
}

# TODO: move to Source::Package ?
use Source::Package::Comparator;
sub get_comparator {
    my $self=shift;
    my $comparator=$self->{-comparator};
    unless ($comparator) {
	my $package_class=$self->get_package_class();
	$comparator=Source::Package::Comparator->new(
	    -epoch_comparator => $package_class->get_raw_epoch_comparator,
	    -version_comparator=>$package_class->get_raw_version_comparator,
	    -release_comparator=>$package_class->get_raw_release_comparator,
	    );
	$self->{-comparator}=$comparator;
    }
    return $comparator;
}

# used in Source/Repository/Mass.pm for --all option
sub all_names_ref {
    my ($self)=@_;
    my $names_ref=$self->{-names};
    return $names_ref ? $names_ref : [];
}

sub name2pkg {
    my ($self,$name)=@_;
    #print STDERR "try to resolve name: $name\n";
    my $name2pkg=$self->{-name2pkg};
    if ($name2pkg) {
	return $name2pkg->{$name};
    }
    return;
}

# sometime, fix in Mass and remove
sub is_pkg {
   my ($self,$item)=@_;
   return $self->get_package_class->is_pkg($item);
}

# sometime, fix in Mass and remove
sub file2pkg {
    my ($self,$file)=@_;
    return $self->get_package_class->file2pkg($file);
}

#------------------------------------------
#    private methods
#------------------------------------------

#------------------------------------------
#    protected methods
#------------------------------------------

sub __load_repolist {
    my $self=shift;
    my $list=$self->{-repolist};
    return unless defined $list;
    print STDERR "DEBUG:__load_repolist: -repolist=> $list\n" if $verbose>2;
    # TODO: baseurl support
    ($self->{-names}, $self->{-name2pkg})=&__load_list($list,\&__default_list3_processor,$self->get_package_class,$self->{-cachedir});
}

sub __load_namelist {
    my $self=shift;
    my $list=$self->{-namelist};
    return unless defined $list;
    print STDERR "DEBUG:__load_namelist: -namelist=> $list\n" if $verbose>2;
    ($self->{-names})=&__load_list($list);
}

#------------------------------------------
#    private utilities
#------------------------------------------

sub __split_path_like_uri_by_colon {
    my $paths=shift;
    my @prepath;
    foreach (split(/(?<!\\):/,$paths)) {
	# support for paths like widehat.opensuse.org/repositories/games\:/unknown-horizons
	s!\\:!:!g;
	if ($_ eq '') {
	    $_ = '.';
	} elsif (/^~/) {
	    s/^~/$ENV{HOME}/;
	}
	#warn "__split_path_like_uri_by_colon: [$_]\n";
	push @prepath, $_;
    }
    my @path;
    while (scalar @prepath) {
	my $element=shift @prepath;
	if (
	    ($element eq 'http' or
	     $element eq 'https' or
	     $element eq 'ftp' or
	     $element eq 'rsync') and @prepath>0 and substr($prepath[0],0,2) eq '//'
	    ) {
	    push @path, $element.':'.(shift @prepath);
	} elsif ($element eq '.' and scalar @path and $path[$#path] =~ m!^rsync://!) {
	    $path[$#path] .= '::'.(shift @prepath);
	} else {
	    push @path, $element;
	}
    }
    return @path;
}

#------------------------------------------
#    private utilities
#------------------------------------------

sub __load_list {
    my $list=shift;
    my $processor=shift;
    return unless defined $list;
    my @all_names;
    my %name2pkg;
    open (my $fn, '<', $list) or die "can't open file $list\n";
    while (my $line=<$fn>) {
	my @list = &_line2array($line);
	next unless @list;
	my $name=$list[0];
	push @all_names, $name;
	$name2pkg{$name}=&$processor(\@list,@_) if $processor;
    }
    close($fn);
    return \@all_names, \%name2pkg;
}

sub __default_list3_processor {
    my ($ref,$class,$cachedir)=@_;
    return unless $ref;
    return unless @$ref;
    my ($name, $versionspec, $url)=@$ref;
    my @args=(
	-name => $name,
	-url => $url,
	);
    my ($epoch,$release);
    if ($versionspec=~s/^(\d+)://) {
	push @args, -epoch=> $1;
    }
    if ($versionspec=~s/-([^-]+)$//) {
	push @args, -release=> $1;
    }
    push @args, -cachedir=> $cachedir if $cachedir;
    push @args, -version => $versionspec;
    return $class->new(@args);
}

1;
