package Source::Repository::RPM;

use strict;
use warnings;
use Carp;
use RPM::Vercmp;
use File::Find;
use Source::Package::RPM;
use Source::Repository::Empty;
use Source::Repository;
our @ISA=qw/Source::Repository/;

our $VERSION=0.001;
our $verbose=0;

sub get_package_class {
    return 'Source::Package::RPM';
}

sub create_repo_from_dir {
    my ($class, $opt, $uri)=@_;
    my %name2file;
    my %name2vr;
    Carp::confess "not a dir: $uri" if ! -d $uri;
    my $file_validator_sub=$opt->{-file_validator};
    $file_validator_sub||=\&__is_valid_src_rpm_file;
    find (
	sub {
	    &__add_srpm($file_validator_sub,\%name2vr,\%name2file,$_,$File::Find::name);
	}, $uri);
    return $class->new(%$opt, -name2file => \%name2file, -created_from => $uri );
}

sub create_repo_from_url {
    my ($class, $opt, $url)=@_;
    my %name2file;
    my %name2vr;
    my $file_validator_sub=$opt->{-file_validator};
    $file_validator_sub||=\&__is_valid_src_rpm_file;
    my $name2url=&__load_url($file_validator_sub, $url);#,$opt->{-cachedir}
    if (%$name2url) {
	return $class->new(
	    %$opt,
	    -name2url => $name2url,
	    -created_from => $url,
	    );
    } else {
	warn $opt->{-reponame} ? $opt->{-reponame}: 'info',": URL $url: empty list, skipped\n";
	return Source::Repository::Empty->new(%$opt);
    }
}

sub all_names_ref {
    my ($self)=@_;
    my $names=$self->{-names};
    if (!$names) {
      SEARCH:
	foreach my $namekey (qw/-name2file -name2url -name2evr -name2suffix/) {
	    my $nameval=$self->{$namekey};
	    if ($nameval) {
		$names=$self->{-names}=[keys(%$nameval)];
		last SEARCH;
	    }
	}
    }
    return $names;
}

sub name2pkg {
    my ($self,$name)=@_;
    Carp::croak "name2pkg: got undef for name" if ! defined($name);
    my $name2file=$self->{-name2file};
    if ($name2file) {
	my $rpmfile=$name2file->{$name};
	return unless $rpmfile;
	return $self->get_package_class->file2pkg($rpmfile);
    }
    my $name2url=$self->{-name2url};
    if ($name2url) {
	my $url=$name2url->{$name};
	return unless $url;
	return $self->get_package_class->url2pkg($url, $self->{-cachedir});
    }
    my $name2pkg=$self->{-name2pkg};
    if ($name2pkg) {
	return $name2pkg->{$name};
    }
    #use Data::Dumper;
    #print Dumper($self);
    die "Oops! name2pkg not set up properly.";
}

### TODO: move to Source::Package subclass?
### used in Mass::Transform::Embedded and Mass::BundleImport
### class method
sub pkg4rpmeditor {
    my ($class, $pkg)=@_;
    return [ SOURCERPM => $pkg->file];
}

#------------------------------------------
#    private utilities
#------------------------------------------

# returns 1, name, version-release or undef
sub __is_valid_src_rpm_file {
    if ($_[0]=~/^(.*)-([^-]*-[^-]*)\.src\.rpm$/) {
	return (1,$1,$2);
    }
    return;
}

sub __add_srpm {
    my ($file_validator_sub,$name2vr,$name2path,$in,$value,$prefix)=@_;
    my ($is_valid_file, $name, $vr)= &$file_validator_sub($in);
    if ($is_valid_file) {
	my $oldvr=$name2vr->{$name};
	if (!defined($oldvr) or RPM::Vercmp::evrcmp($oldvr, $vr)<=0) {
	    $name2vr->{$name}=$vr;
	    if (defined $value) {
		$name2path->{$name}=$value;
	    } else {
		$name2path->{$name}=$prefix.$in;
	    }
	}
    }
}

sub __load_url {
    my ($file_validator_sub, $url)=@_;
    my %name2url;
    my %name2vr;
    $url.='/' unless substr($url, -1, 1) eq '/';
    if ($url =~ m!^rsync://!) {
	&__deep_rsync($file_validator_sub, $url,\%name2vr, \%name2url);
    } else {
	&__deep_wget($file_validator_sub, $url,\%name2vr, \%name2url);
    }
    return \%name2url;
}

sub __deep_rsync {
    my ($file_validator_sub, $url,$name2vr, $name2url)=@_;
    my $rsync_url=$url;
    $rsync_url =~ s!^rsync://!! if $rsync_url =~ /::/;
    my $cmd="rsync --list-only $rsync_url |";
    open (my $fn, $cmd) or die "can't open $cmd\n";
    while (my $line=<$fn>) {
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=5;
	if (substr($list[0],0,1) eq '-') {
	    &__add_srpm($file_validator_sub, $name2vr, $name2url, $list[4], undef, $url);
	} elsif (substr($list[0],0,1) eq 'd' and $list[4]=~/^[_0-9a-zA-Z]$/) {
	    # fc tree support
	    &__deep_rsync($file_validator_sub, $url.$list[4].'/',$name2vr, $name2url);
	}
    }
    close($fn);
}

sub __deep_wget {
    my ($file_validator_sub, $url,$name2vr, $name2url)=@_;
    my $cmd="wget -q -c -O- $url |";
    my $content=_read_file($cmd);
    my @urls=$content=~m/\<a\s+href="?([^"\s>]+)"?\s*>/gi;
    if (@urls) {
	map {print "$_\n"} @urls if $verbose>2;
	map {&__add_srpm($file_validator_sub, $name2vr, $name2url, $_, undef, $url)} @urls;
    } else {
	# fc tree support
	my @subdirs=$content=~m!\<a\s+href="?([_0-9a-z]/)"?\s*>!gi;
	map {&__deep_wget($file_validator_sub, $url.$_, $name2vr, $name2url)} @subdirs;
    }
}

sub _read_file {
    my $file=shift;
    my $content = '';
    open my $fh, $file or die $!;
    {
	local $/;
	$content = <$fh>;
    }
    close $fh;
    return $content;
}



1;

__END__
