package Source::Repository::RPM::ALTLinuxSrcList;

use strict;
use warnings;
use RPM::Vercmp;

use ALTLinux::RepoList 0.006 qw/load_listfile_lambda_fn listdirs2srclists is_listuri_in_repo_tree listuri2reporoot/;
use Source::Shared::Utils::Downloader qw/named_cache_download/;
use Source::Package::RPM::ALT;
use Source::Repository::RPM;
our @ISA=qw/Source::Repository::RPM/;

our $VERSION=0.002;
our $verbose=0;

# Note: if using alt src list,
# does not need src.rpms to be actually present

# Note: enable to allow hashertar format .tar files too
our $allow_hashertar=0;
our $package_class='Source::Package::RPM::ALT';

sub get_package_class {
    return $package_class;
}

sub create_repo_from_dir {
    my ($class, $opt, $dir)=@_;
    $dir .= '/' if $dir !~ m!/$!;
    my $listdir=$dir.'../list';
    my ($altsrclist)=&listdirs2srclists($listdir) if -d $listdir;
    #warn "DEBUG: listdir=$listdir altsrclist=$altsrclist\n" if $verbose>2;
    if ($altsrclist and -e $altsrclist) {
	my ($names, $name2evr, $name2suffix)=&load_listfile_lambda_fn($altsrclist, \&__load_altsrclist_fn);
	return $class->new(
	    -fileprefix => $dir,
	    -created_from => $dir,
	    %$opt,
	    -names => $names,
	    -name2evr => $name2evr,
	    -name2suffix => $name2suffix,
	    );
    } else {
	$opt->{-file_validator}=\&__is_valid_alt_src_file if $allow_hashertar;
	return $class->SUPER::create_repo_from_dir($opt, $dir);
    }
}

sub create_repo_from_file {
    my ($class, $opt, $altsrclist)=@_;
    if (!&is_listuri_in_repo_tree($altsrclist)) {
	$opt->{-file_validator}=\&__is_valid_alt_src_file if $allow_hashertar;
	return $class->SUPER::create_repo_from_file($opt, $altsrclist);
    } else {
	my $dir=&listuri2reporoot($altsrclist).'/files/SRPMS/';
	my ($names, $name2evr, $name2suffix)=&load_listfile_lambda_fn($altsrclist, \&__load_altsrclist_fn);
	return $class->new(
	    -fileprefix => $dir,
	    -created_from => $altsrclist,
	    %$opt,
	    -names => $names,
	    -name2evr => $name2evr,
	    -name2suffix => $name2suffix,
	    );
    }
}

sub create_repo_from_url {
    my ($class, $opt, $altsrclist_url)=@_;
    if (!&is_listuri_in_repo_tree($altsrclist_url)) {
	print STDERR "DEBUG: not a src.list: $altsrclist_url\n" if $verbose >2;
	return $class->SUPER::create_repo_from_url($opt, $altsrclist_url);
    } else {
	my $dir=&listuri2reporoot($altsrclist_url).'/files/SRPMS/';
	my ($names, $name2evr, $name2suffix)=&__load_altsrclist_url($altsrclist_url,$opt->{-cachedir});
	die "bad URL $altsrclist_url: empty list" if ! %$name2suffix;
	return $class->new(
	    -urlprefix => $dir,
	    -created_from => $altsrclist_url,
	    %$opt,
	    -names => $names,
	    -name2evr => $name2evr,
	    -name2suffix => $name2suffix,
	    );
    }
}

sub name2pkg {
    my ($self,$name)=@_;
    my $name2suffix=$self->{-name2suffix};
    return $self->SUPER::name2pkg($name) if ! $name2suffix;
    my $suffix=$name2suffix->{$name};
    return unless $suffix;
    my $evr=$self->{-name2evr}->{$name};
    if ($evr=~m!^(?:(\d+):)?([^-]+)-(.*)$!) {
	my $prefix=$self->{-fileprefix};
	my $urlprefix=$self->{-urlprefix};
	if ($urlprefix) {
	    return $self->get_package_class->new(
		-name => $name,
		-epoch=> $1,
		-version => $2,
		-release => $3,
		-cachedir => $self->{-cachedir},
		-url => $urlprefix.$suffix,
		);
	} else {
	    return $self->get_package_class->new(
		-name => $name,
		-epoch=> $1,
		-version => $2,
		-release => $3,
		-file => $prefix.$suffix,
		);
	}
    } else {
	die 'oops! ',__PACKAGE__, ': bad evr: ',$evr;
    }
}

#------------------------------------------
#    private utilities
#------------------------------------------

# returns 1, name, version-release or undef
# this variant allows hasher tar and src.rpm
sub __is_valid_alt_src_file {
    if ($_[0]=~/^(.*)-([^-]*-[^-]*)\.(?:src\.rpm|tar)$/) {
	return (1,$1,$2);
    }
    return;
}

sub __load_altsrclist_url {
    my ($url,$cachedir)=@_;
    if ($url =~ s!^rsync://!!) {
	#my $altsrclist=&__get_rsync_url_cache($cachedir,'altsrclist',$url);
	my $altsrclist=named_cache_download($url,'altsrclist',$cachedir);
	return &load_listfile_lambda_fn($altsrclist, \&__load_altsrclist_fn);
    } else {
	my $cmd="wget -q -c -O- $url |";
	open (my $fn, $cmd) or die "can't open $cmd\n";
	my @ret=&__load_altsrclist_fn($fn);
	close($fn);
	return @ret;
    }
}

sub __load_altsrclist_fn {
    my ($fn)=@_;
    my @all_names;
    my %name2evr;
    my %name2suffix;
    while (my $line=<$fn>) {
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=3;
	my ($name,$evr,$suffix)=@list;
	my $oldevr=$name2evr{$name};
	if (! defined($oldevr)) {
	    push @all_names, $name;
	    $name2evr{$name}=$evr;
	    $name2suffix{$name}=$suffix;
	} elsif (RPM::Vercmp::evrcmp($oldevr, $evr)<=0) {
	    $name2evr{$name}=$evr;
	    $name2suffix{$name}=$suffix;
	}
    }
    return \@all_names, \%name2evr, \%name2suffix;
}

1;

__END__
